﻿# Setze das Security Protocol auf TLS 1.2
[Net.ServicePointManager]::SecurityProtocol = [Net.SecurityProtocolType]::Tls12

# Parameter für die E-Mail Benachrichtigung
$smtpServer = "smtp.xxx.com"
$smtpPort = 587
$from = "xxx@xxx.de"
$to = "xxx@xxx.de"
$subject = "Snapshot Alert - $env:COMPUTERNAME"  # Dynamischer Servername
$smtpUser = "xxx@xxx.de"  # Dein SMTP-Benutzername
$smtpPassword = "xxx"  # Dein SMTP-Passwort

# Bereite die Anmeldeinformationen vor
$secPass = ConvertTo-SecureString $smtpPassword -AsPlainText -Force
$credential = New-Object System.Management.Automation.PSCredential ($smtpUser, $secPass)

# Verzeichnisse, die durchsucht werden sollen (Mehrere Pfade als Array)
$hyperVPaths = @("D:\", "E:\")

# Log-Datei Pfad
$logFilePath = "C:\Services\Logs\AVHDX-Script.log"

# Funktion zum Schreiben in die Logdatei
function Write-Log {
    param (
        [string]$logMessage
    )

    $timestamp = Get-Date -Format "yyyy-MM-dd HH:mm:ss"
    $logEntry = "$timestamp - $logMessage"

    # Logeintrag in die Datei schreiben
    $logEntry | Add-Content -Path $logFilePath
}

# Logdatei auf 14 Tage begrenzen
function Cleanup-LogFile {
    if (Test-Path -Path $logFilePath) {
        $logEntries = Get-Content $logFilePath
        $thresholdDate = (Get-Date).AddDays(-14)

        # Nur gültige Log-Einträge filtern
        $filteredLogEntries = $logEntries | Where-Object {
            if ($_ -match '^\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}') {
                try {
                    $logDate = [datetime]($_.Split(' - ')[0])
                    return $logDate -ge $thresholdDate
                } catch {
                    return $false
                }
            } else {
                return $false
            }
        }

        # Gefilterte Einträge zurückschreiben
        $filteredLogEntries | Set-Content -Path $logFilePath
        Write-Log "Logdatei bereinigt: Ältere Einträge als 14 Tage wurden entfernt."
    }
}

# Funktion zum Versenden der E-Mail
function Send-AlertMail {
    param (
        [string]$filePath,
        [string]$vmName,
        [string]$partitionSize,
        [string]$freeSpace,
        [string]$fileSize,
        [string]$creationDate
    )

    # HTML-Inhalt der E-Mail
    $body = @"
<html>
<head>
    <style>
        body { font-family: Arial, sans-serif; font-size: 14px; }
        table { border-collapse: collapse; width: 100%; }
        table, th, td { border: 1px solid black; padding: 8px; text-align: left; }
        th { background-color: #f2f2f2; }
    </style>
</head>
<body>
    <p>Auf dem Server <b>$env:COMPUTERNAME</b> wurde eine <b>.avhdx</b> Datei gefunden, die älter als 2 Tage ist.</p>
    <table>
        <tr>
            <th>Detail</th>
            <th>Information</th>
        </tr>
        <tr>
            <td><b>Pfad</b></td>
            <td>$filePath</td>
        </tr>
        <tr>
            <td><b>Virtueller Server</b></td>
            <td>$vmName</td>
        </tr>
        <tr>
            <td><b>Snapshotgröße</b></td>
            <td>$fileSize</td>
        </tr>
        <tr>
            <td><b>Belegter Speicher</b></td>
            <td>$partitionSize</td>
        </tr>
        <tr>
            <td><b>Freier Speicherplatz</b></td>
            <td>$freeSpace</td>
        </tr>
        <tr>
            <td><b>Erstellungsdatum</b></td>
            <td>$creationDate</td>
        </tr>
    </table>
</body>
</html>
"@

    # E-Mail erstellen und senden
    $mailMessage = New-Object System.Net.Mail.MailMessage
    $mailMessage.From = $from
    $mailMessage.To.Add($to)
    $mailMessage.Subject = $subject
    $mailMessage.Body = $body
    $mailMessage.IsBodyHtml = $true
    $mailMessage.BodyEncoding = [System.Text.Encoding]::UTF8

    $smtpClient = New-Object System.Net.Mail.SmtpClient($smtpServer, $smtpPort)
    $smtpClient.EnableSsl = $true
    $smtpClient.Credentials = $credential

    try {
        $smtpClient.Send($mailMessage)
        Write-Log "E-Mail gesendet für Datei: $filePath (VM: $vmName, Dateigröße: $fileSize GB, Freier Speicher: $freeSpace GB, Partitionsgröße: $partitionSize GB)"
    } catch {
        Write-Log "Fehler beim Senden der E-Mail: $_"
        throw $_
    } finally {
        $mailMessage.Dispose()
        $smtpClient.Dispose()
    }
}

# Überprüfen, ob der Ordner C:\Services existiert, und erstellen, wenn er nicht vorhanden ist
if (-not (Test-Path -Path "C:\Services")) {
    New-Item -Path "C:\Services" -ItemType Directory
}

# Logdatei initialisieren (erstellt eine neue Datei, falls sie nicht existiert)
if (-not (Test-Path -Path $logFilePath)) {
    New-Item -Path $logFilePath -ItemType File -Force
}

# Log schreiben, dass das Skript gestartet wurde
Write-Log "Skriptstart: Suche nach AVHDX/AVHD Dateien gestartet."

# Bereinigung der Logdatei
Cleanup-LogFile

# Schleife durch alle Hyper-V Pfade
$foundFiles = $false
foreach ($hyperVPath in $hyperVPaths) {
    # Alle Unterverzeichnisse durchsuchen
    Get-ChildItem -Path $hyperVPath -Recurse | Where-Object {
        $_.Extension -eq ".avhdx" -or $_.Extension -eq ".avhd"
    } | ForEach-Object {
        $file = $_
        $creationDate = $file.CreationTime

        # Prüfen, ob die Datei älter als 2 Tage ist
        if ($creationDate -lt (Get-Date).AddDays(-2)) {
            $filePath = $file.FullName
            $fileSize = [math]::Round($file.Length / 1GB, 2) # Größe in GB

            # Holen der VM-Informationen mit Get-VM und Zuordnung der Festplatte
            $vm = Get-VM | ForEach-Object {
                $vmHardDisks = Get-VMHardDiskDrive -VM $_
                if ($vmHardDisks) {
                    foreach ($disk in $vmHardDisks) {
                        if ($disk.Path -eq $filePath) {
                            return $_ # VM zurückgeben
                        }
                    }
                }
            }

            if ($vm) {
                $vmName = $vm.Name
            } else {
                $vmName = "Unbekannt, warscheinlich Überreste"
                Write-Log "Keine Zuordnung für Datei $filePath zu einer VM gefunden."
            }

            # Laufwerksinformationen (Partition und freier Speicherplatz)
            $driveLetter = $filePath.Substring(0, 2)
            $driveInfo = Get-PSDrive -Name $driveLetter.TrimEnd(":")

            $partitionSize = [math]::Round($driveInfo.Used / 1GB, 2) # Gesamtgröße in GB
            $freeSpace = [math]::Round($driveInfo.Free / 1GB, 2) # Freier Speicher in GB

            # Log schreiben, dass eine Datei gefunden wurde
            Write-Log "Gefundene Datei: $filePath (VM: $vmName, Dateigröße: $fileSize GB, Freier Speicher: $freeSpace GB, Partitionsgröße: $partitionSize GB)"
            $foundFiles = $true

            # E-Mail versenden
            Send-AlertMail -filePath $filePath -vmName $vmName -partitionSize "$partitionSize GB" -freeSpace "$freeSpace GB" -fileSize "$fileSize GB" -creationDate $creationDate
        }
    }
}

# Log schreiben, dass das Skript beendet wurde
if ($foundFiles) {
    Write-Log "Skriptende: Suche abgeschlossen. Snapshots gefunden und gemeldet."
    Write-Output "Skript erfolgreich ausgeführt: Snapshots gefunden und gemeldet."
} else {
    Write-Log "Skriptende: Keine Snapshots gefunden."
    Write-Output "Skript erfolgreich ausgeführt: Keine Snapshots gefunden."
}
